'' PropDOS  Propeller MiniDOS V1.6_LCD
'' By Jeff Ledger
'' Thanks to Mike Green & Tomas Rokicki for the TxtComp & modded Fsrw code (from FemtoBASIC)
'' Thanks to Andrew Arsenault for Hydra Sound System

'' ---------------> SEE CON for I/O Settings  <----------------

'' PropDOS_lcd COMMANDS:
'' ---------------------
'' DIR = show files on the SD
'' DIR *.ext = show files with .EXT
'' DEL = delete file (del filename)
'' SPIN = load/run binary from SD (spin filename)
''  [ .BIN files can be executed 'DOS-like' by simply typing the ]
''  [ filename without the .bin extension.                       ]
''
'' CLS = clear the screen
'' TYPE = display a text file (type filename)
'' PLAY = load/play an hmus file (play filename)
'' STOP = stop playing current hmus file
'' RESUME = resume playing currently loaded file
'' REBOOT = Reboot the Propeller
'' COPY CON filename = Simple text file creator, exit with Control-Z on a blank line
'' BON    = LCD Backlight on (default)
'' BOFF   = LCD Backlight off
''
'' Holding [ESC] at startup will bypass PropDOS and excute "MENU.BIN" from the SD card.
'' {filename located around line 150}

CON
  '' I/O SETTINGS

   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000
  
  '' Adjust for typical location of the keyboard. Usually 26 on the demoboard
  
   keyboard  = 26
   LCD_PIN   = 4           ' for Parallax 4x20 serial LCD on P0
   LCD_BAUD  = 19_200
   LCD_LINES = 4 
   
  '' Adjust for location of the SD card.  Typical settings below.
{
  '' Typical Hydra Configuration:
  
   spiDO     = 16
   spiClk    = 17
   spiDI     = 18
   spiCS     = 19
}

  '' Typical Demoboard/Protoboard Configuration
  
   spiDO     = 0
   spiClk    = 1
   spiDI     = 2
   spiCS     = 3

 '' ==== NO CHANGES REQUIRED PAST THIS POINT ====

  '' Key Assignments 

   fReturn   = 13
   bspKey    = $C8
   escape    = $CB
   backspace = 8

  CR                            = $0D
  LF                            = $0A
  TAB                           = $09
  BKS                           = $08


   
OBJ
     key   : "comboKeyboard"             ' Keyboard Driver
     text  : "debug_lcd"
     fsrw  : "fsrwFemto"                 ' Modified SD routines
     hss   : "yma_hss_v1.2"              ' Hydra Sound System 
      
VAR
     byte command[32]
     byte f0[32]
     byte tbuf[20] ' SD Vars
     long ioControl[2] 
     long tp
     byte music[18000]
     byte cline[42]
     byte ext[3]
     byte strbuff[5]
     
PUB mainProgram  | size, r,sta,bytes,counter,newcount,i,n,j,spare,t,Pass,Pos,keyval,tt,rr,SampleRate,Samples  

  hss.start
  key.start(keyboard)
  text.init(LCD_PIN, LCD_BAUD, LCD_LINES) 
  fsrw.start(@ioControl)                         ' Start I2C/SPI driver
  text.backLight(true)
  text.cls  
  text.str(string("PropDOS V1.6 LCD",CR,"By Jeff Ledger",CR,CR))
 

  '' Check for the required keyboard & SD card, gracefull abort if not found.
  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0
     text.str(string(13,"No SD card!",CR))
     text.str(string("Expected on: "))
     text.dec(spiDO)
     text.str(string(", "))
     text.dec(spiClk)
     text.str(string(", "))
     text.dec(spiDI)
     text.str(string(", "))
     text.dec(spiCS)
     text.str(string(13,CR,"System halted.",CR)) 
     abort 
  waitcnt(clkfreq + cnt)
  if not key.present                                
     text.str(string(13,"No keyboard!",CR))
     text.str(string("Expected on: "))
     text.dec(keyboard)
     text.str(string(13,CR,"System halted.",CR))
     abort


  '' Check for the ESC key to bypass PropDOS to selected program

  
  keyval:=key.key
      if keyval == 203
          fsrw.popen(string("menu.bin"),"r")
          fsrw.bootSDCard
    
  repeat
   bytefill(@command,0,32) 'Clear the command buffer
   bytefill(@cline,0,14) ' Clear commandline buffer
   
   text.str(string("SD:>"))
   input(@command,32)

   if textComp(@command,string("spin"),4) == true 
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
    fsrw.popen(@cline,"r")
    fsrw.bootSDCard

   elseif textComp(@command,string("$"),1) == true
     dirw

   elseif textComp(@command,string("bon"),3) == true
    text.cls
    text.backLight(true) 
    text.gotoxy(0,4)
    
   elseif textComp(@command,string("boff"),4) == true
    text.cls
    text.backLight(false) 
    text.gotoxy(0,4)
   elseif textComp(@command,string("dir *."),6) == true
    counter := 6 ' Starting point of command line (after space)
    newcount := 0
    repeat until newcount > 2
       cline[newcount] := command[counter]
       cline[newcount]-=32
       counter++
       newcount++
    text.putc(13)
    text.str(string("Directory of SD:"))
    text.str(string("*."))
    counter := 0
    repeat until counter > 4
      if cline[counter] <> 0
        text.putc(cline[counter])
      counter++
    text.putc(13)
    text.putc(13)
    fsrw.opendir
    repeat while fsrw.nextfile(@tbuf) == 0
      t:=StrRight(@tbuf,3)
      if strcomp(t,@cline)
         text.str(string(" "))
         text.str(@tbuf)
         text.putc(13)
    text.putc(13)

   elseif textComp(@command,string("dir"),3) == true
     dirw
     
   elseif textComp(@command,string("reboot"),6) == true
     reboot

   elseif textComp(@command,string("del"),3) == true
     counter := 4 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     r := fsrw.popen(@cline,"d")
     text.cls
     text.str(string(13,"File deleted.",CR,CR)) 
     text.gotoxy(0,4)
     
   elseif textComp(@command,string("resume"),6) == true
    hss.hmus_play 'Play loaded file. 

   elseif textComp(@command,string("copy con"),4) == true 
     counter := 9 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
     r := fsrw.popen(@cline,"w")
     text.cls
     text.gotoxy(0,0)
     repeat
      bytefill(@cline,0,40) 'Clear the buffer 
      input(@cline,39)
        if textComp(@cline,string("z"),1) == true
          quit        
      counter := 0
      repeat 
         r := (cline[counter]) 
           if cline[counter] <> 0
             fsrw.out(r)

         counter++ 
         if counter == 39
           fsrw.out(13)
           quit

   
   elseif textComp(@command,string("play"),4) == true 
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     r := fsrw.popen(@cline,"r")
      repeat
        r := fsrw.pgetc
          if r < 0 or counter > 7999
            'play file
             hss.hmus_load(@music)'load Hmus file into player.
             hss.hmus_play 'Play loaded file.
             text.putc(13)
             text.str(string("<"))
             text.str(string("**"))
             text.str(string("> Playing: "))
             counter := 0
             repeat until counter > 14
              if cline[counter] <> 0
               text.putc(cline[counter])
              counter++
             text.str(string(" <"))
             text.str(string("**"))
             text.str(string(">",CR))
             text.str(string("using Hydra Sound System v1.2",CR))
             text.str(string("By Andrew Arsenault",CR,CR))
             quit
        music[counter] := r
        counter++

   
     
   elseif textComp(@command,string("cls"),3) == true
     text.cls
     text.gotoxy(0,0) 

   elseif textComp(@command,string("stop"),4) == true
     hss.hmus_stop
     text.gotoxy(0,4)

   elseif textComp(@command,string("type"),4) == true
     text.cls
     text.gotoxy(0,0)
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
     r := fsrw.popen(@cline,"r")  
      repeat
        r := fsrw.pgetc
         if r < 0
           text.putc(13)
           quit
        if r > 0
          text.putc(r)

   elseif textComp(@command,string("ver"),3) == true
      text.cls
      text.str(string("PropDOS V1.6_lcd",13,"By Jeff Ledger",13,10))
      text.gotoxy(0,4)
      
   elseif command[1] <> 0    '' AUTO-EXECUTE A .BIN FILE LIKE DOS
      counter := 0
      repeat until counter > 13
         if command[counter] > 0
            '' Convert Lowercase to UPPERCASE
            if command[counter] > 96 and command[counter] < 123
               command[counter] := command[counter]-=32
            cline[counter] := command[counter]
         counter++ 

      counter:=strsize(@cline)
      counter--
      counter++                   '' AAAAAARRRRRRGGG!!!
      cline[counter]:=46          '' This paticular routine
      counter++                   '' proved to be a bit challenging.
      cline[counter]:=66          '' To allow .BIN's to execute,
      counter++                   '' while ignoring everything else.
      cline[counter]:=73          '' --works perfectly.. :)
      counter++
      cline[counter]:=78
      rr:=strsize(@cline)
      fsrw.opendir
      repeat while fsrw.nextfile(@tbuf) == 0
        tt:=(StrRight(@tbuf,3))
        if textcomp(tt,string("BIN"),3) == true
           if textComp(@cline,@tbuf,rr) == true
              fsrw.popen(@cline,"r")
              fsrw.bootSDCard

        
PUB dirw | r, sta, bytes,b,d,tt,hidden

   text.cls
   text.gotoxy(0,0)
 
   fsrw.opendir                            
   b := 0
   hidden:= false                                  
   d := false                              
   repeat while fsrw.nextfile(@tbuf) == 0
      tt:=(StrRight(@tbuf,3))
      if textcomp(tt,string("HID"),3) == true
         hidden:=true
      if hidden == false
        d := true                            
        if b == 19                         '' Grepped from FemtoBASIC
           text.putc(13)                    '' Nice Wide Directory Listing!
           b := 0                                 
        text.str(@tbuf)
        text.putc(32)                    
        b := b + 13
      if hidden == true
         hidden := false                           
   if d                                    
      text.putc(13)

          
PUB input(data, size) | keystroke,len
   keystroke:=0
   len:=0
   repeat' until keystroke == 13 'Exit when Carriage Return is pressed
     keystroke := key.key

      if keystroke == 208
         keystroke:=36
     
      if keystroke == $C8       ' Check for Backspace Key
        if len > 0
         byte[data--] := 0
         size++
         len--
         text.putc(8)
         text.putc(32) 'These lines erase the cursor on backspace
         text.putc(8)

         keystroke := 0

      if keystroke == $C8        'Clean up backspace if all is cleared
         keystroke:= 0

      if keystroke == 13
         text.putc(13)
         quit
         
      if keystroke > 10 
        if size > 1

          text.putc(keystroke)
 
          ''text.dec(keystroke) 'for troubleshooting
          if keystroke <> 13 
            byte[data++] := keystroke
            size--
            len++

          if keystroke==36
             text.putc(8)
             text.putc(8)
             text.str(string("dir/w"))
             text.putc(13)
             quit

PRI textComp(str1,str2,len)
   repeat len
      if byte[str1++] <> byte[str2++]
         return false
   return true

PUB StrRight(str,len) | i,l,z,j

  l:=strSize(str)
  z:=0
  repeat i from l-len to l
    strbuff[z] := byte[@tbuf][i]
    z++
  return @strbuff

 